--------------------------------------------------------------
--! @file detector_glingon_tb.vhd
--! @brief Testbench para o detector de transmissão em Glingon
--! @author Edson Midorikawa (emidorik@usp.br)
--! @date 2020-06-01
--------------------------------------------------------------

library ieee;
use ieee.numeric_bit.all;
use ieee.math_real.all;

--  A testbench has no ports.
entity detector_glingon_tb is
end entity detector_glingon_tb;

architecture testbench of detector_glingon_tb is

  --  Declaration of the component to be tested.  
  component detector_glingon
  port (
        CLOCK : in bit;
        RESET : in bit;
        STX :   in bit;
        SIN :   in bit;
        SOUT :  out bit_vector(1 downto 0);
        ACK :   out bit
      );
  end component detector_glingon;
  
  -- Declaration of signals
  signal s_reset : bit;
  signal s_clock : bit;  
  signal s_stx   : bit;
  signal s_sin   : bit;
  signal s_sout  : bit_vector(1 downto 0);
  signal s_ack   : bit;

begin
  -- Component instantiation
  -- DUT = Device Under Test 
  DUT: entity work.detector_glingon(dataflow) port map (CLOCK => s_clock,
                                              RESET => s_reset,
                                              STX => s_stx,
                                              SIN => s_sin,
                                              SOUT => s_sout,
                                              ACK => s_ack);

  -- Clock generator
  clk: process is
  begin
    s_clock <= '0';
    wait for 0.5 ns;
    s_clock <= '1';
    wait for 0.5 ns;
  end process clk;  

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      reset : bit;
      stx   : bit;
      sin   : bit;
      
      --  The expected outputs of the circuit.
      sout  : bit_vector(1 downto 0);
      ack   : bit;
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ('1','0','0',"11",'0'),  --  1. reset
        ('0','1','0',"00",'1'),  --  2. STX=1 e 0 * K
        ('0','0','0',"00",'1'),  --  3. 0 * K
        ('0','0','1',"11",'0'),  --  4. 1
        ('0','0','1',"10",'1'),  --  5. 1 * W
        ('0','0','1',"11",'0'),  --  6. 1
        ('0','0','0',"01",'1'),  --  7. 0 * Z
        ('0','0','1',"11",'0'),  --  8. 1 
        ('0','0','0',"01",'1'),  --  9. 0 * Z
        ('0','0','1',"11",'0'),  -- 10. 1
        ('0','0','1',"10",'1'),  -- 11. 1 * W
        ('0','0','0',"00",'1'),  -- 12. 0 * K
        ('0','0','0',"00",'1'),  -- 13. 0 * K
        ('0','0','1',"11",'0'),  -- 14. 1
        ('0','0','0',"01",'1'),  -- 15. 0 * Z
        ('0','0','1',"11",'0'),  -- 16. 1
        ('0','0','1',"10",'1'),  -- 17. 1 * W
        ('1','0','0',"11",'0'),  -- 18. reset
        ('0','1','1',"11",'0'),  -- 19. STX=1 e 1
        ('0','0','0',"01",'1'),  -- 20. 0 * Z
        ('0','0','0',"00",'1'),  -- 21. 0 * K
        ('0','0','1',"11",'0'),  -- 22. 1
        ('0','0','1',"10",'1'),  -- 23. 1 * W
        ('0','0','1',"11",'0'),  -- 24. 1
        ('0','0','1',"10",'1'),  -- 25. 1 * W
        ('1','0','0',"11",'0'),  -- 26. reset
        ('0','1','1',"11",'0'),  -- 27. STX=1 e 1
        ('0','0','1',"10",'1'),  -- 28. 1 * W
        ('0','0','0',"00",'1'),  -- 29. 0 * K
        ('0','0','0',"00",'1'),  -- 30. 0 * K
        ('0','0','1',"11",'0'),  -- 31. 1
        ('0','0','0',"01",'1')); -- 32. 0 * Z
        
  begin
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.      
      s_reset <= patterns(k).reset;
      s_stx   <= patterns(k).stx;
      s_sin   <= patterns(k).sin;
      
      --  Wait for the results.
      wait for 1 ns;
      
      assert s_sout = patterns(k).sout
        report "Invalid output for sout" severity error;

      assert s_ack = patterns(k).ack
        report "Invalid output for ack" severity error;
    end loop;

    -- report "Fim da simulação!";
    assert false report "End of test" severity note;        
    wait; -- End simulation
  end process stimulus_process;
end architecture testbench;
  

